/* This file was generated by MKnames.awk */

#include <curses.priv.h>

#define IT NCURSES_CONST char * const


#if BROKEN_LINKER || USE_REENTRANT

static const char _nc_name_blob[] = 
"bw\0" "auto_left_margin\0" "am\0" "auto_right_margin\0" "xsb\0" "no_esc_ctlc\0" \
"xhp\0" "ceol_standout_glitch\0" "xenl\0" "eat_newline_glitch\0" "eo\0" \
"erase_overstrike\0" "gn\0" "generic_type\0" "hc\0" "hard_copy\0" "km\0" \
"has_meta_key\0" "hs\0" "has_status_line\0" "in\0" "insert_null_glitch\0" \
"da\0" "memory_above\0" "db\0" "memory_below\0" "mir\0" "move_insert_mode\0" \
"msgr\0" "move_standout_mode\0" "os\0" "over_strike\0" "eslok\0" "status_line_esc_ok\0" \
"xt\0" "dest_tabs_magic_smso\0" "hz\0" "tilde_glitch\0" "ul\0" "transparent_underline\0" \
"xon\0" "xon_xoff\0" "nxon\0" "needs_xon_xoff\0" "mc5i\0" "prtr_silent\0" \
"chts\0" "hard_cursor\0" "nrrmc\0" "non_rev_rmcup\0" "npc\0" "no_pad_char\0" \
"ndscr\0" "non_dest_scroll_region\0" "ccc\0" "can_change\0" "bce\0" "back_color_erase\0" \
"hls\0" "hue_lightness_saturation\0" "xhpa\0" "col_addr_glitch\0" "crxm\0" \
"cr_cancels_micro_mode\0" "daisy\0" "has_print_wheel\0" "xvpa\0" "row_addr_glitch\0" \
"sam\0" "semi_auto_right_margin\0" "cpix\0" "cpi_changes_res\0" "lpix\0" \
"lpi_changes_res\0" "cols\0" "columns\0" "it\0" "init_tabs\0" "lines\0" \
"lines\0" "lm\0" "lines_of_memory\0" "xmc\0" "magic_cookie_glitch\0" "pb\0" \
"padding_baud_rate\0" "vt\0" "virtual_terminal\0" "wsl\0" "width_status_line\0" \
"nlab\0" "num_labels\0" "lh\0" "label_height\0" "lw\0" "label_width\0" \
"ma\0" "max_attributes\0" "wnum\0" "maximum_windows\0" "colors\0" "max_colors\0" \
"pairs\0" "max_pairs\0" "ncv\0" "no_color_video\0" "bufsz\0" "buffer_capacity\0" \
"spinv\0" "dot_vert_spacing\0" "spinh\0" "dot_horz_spacing\0" "maddr\0" \
"max_micro_address\0" "mjump\0" "max_micro_jump\0" "mcs\0" "micro_col_size\0" \
"mls\0" "micro_line_size\0" "npins\0" "number_of_pins\0" "orc\0" "output_res_char\0" \
"orl\0" "output_res_line\0" "orhi\0" "output_res_horz_inch\0" "orvi\0" \
"output_res_vert_inch\0" "cps\0" "print_rate\0" "widcs\0" "wide_char_size\0" \
"btns\0" "buttons\0" "bitwin\0" "bit_image_entwining\0" "bitype\0" "bit_image_type\0" \
"cbt\0" "back_tab\0" "bel\0" "bell\0" "cr\0" "carriage_return\0" "csr\0" \
"change_scroll_region\0" "tbc\0" "clear_all_tabs\0" "clear\0" "clear_screen\0" \
"el\0" "clr_eol\0" "ed\0" "clr_eos\0" "hpa\0" "column_address\0" "cmdch\0" \
"command_character\0" "cup\0" "cursor_address\0" "cud1\0" "cursor_down\0" \
"home\0" "cursor_home\0" "civis\0" "cursor_invisible\0" "cub1\0" "cursor_left\0" \
"mrcup\0" "cursor_mem_address\0" "cnorm\0" "cursor_normal\0" "cuf1\0" "cursor_right\0" \
"ll\0" "cursor_to_ll\0" "cuu1\0" "cursor_up\0" "cvvis\0" "cursor_visible\0" \
"dch1\0" "delete_character\0" "dl1\0" "delete_line\0" "dsl\0" "dis_status_line\0" \
"hd\0" "down_half_line\0" "smacs\0" "enter_alt_charset_mode\0" "blink\0" \
"enter_blink_mode\0" "bold\0" "enter_bold_mode\0" "smcup\0" "enter_ca_mode\0" \
"smdc\0" "enter_delete_mode\0" "dim\0" "enter_dim_mode\0" "smir\0" "enter_insert_mode\0" \
"invis\0" "enter_secure_mode\0" "prot\0" "enter_protected_mode\0" "rev\0" \
"enter_reverse_mode\0" "smso\0" "enter_standout_mode\0" "smul\0" "enter_underline_mode\0" \
"ech\0" "erase_chars\0" "rmacs\0" "exit_alt_charset_mode\0" "sgr0\0" "exit_attribute_mode\0" \
"rmcup\0" "exit_ca_mode\0" "rmdc\0" "exit_delete_mode\0" "rmir\0" "exit_insert_mode\0" \
"rmso\0" "exit_standout_mode\0" "rmul\0" "exit_underline_mode\0" "flash\0" \
"flash_screen\0" "ff\0" "form_feed\0" "fsl\0" "from_status_line\0" "is1\0" \
"init_1string\0" "is2\0" "init_2string\0" "is3\0" "init_3string\0" "if\0" \
"init_file\0" "ich1\0" "insert_character\0" "il1\0" "insert_line\0" "ip\0" \
"insert_padding\0" "kbs\0" "key_backspace\0" "ktbc\0" "key_catab\0" "kclr\0" \
"key_clear\0" "kctab\0" "key_ctab\0" "kdch1\0" "key_dc\0" "kdl1\0" "key_dl\0" \
"kcud1\0" "key_down\0" "krmir\0" "key_eic\0" "kel\0" "key_eol\0" "ked\0" \
"key_eos\0" "kf0\0" "key_f0\0" "kf1\0" "key_f1\0" "kf10\0" "key_f10\0" \
"kf2\0" "key_f2\0" "kf3\0" "key_f3\0" "kf4\0" "key_f4\0" "kf5\0" "key_f5\0" \
"kf6\0" "key_f6\0" "kf7\0" "key_f7\0" "kf8\0" "key_f8\0" "kf9\0" "key_f9\0" \
"khome\0" "key_home\0" "kich1\0" "key_ic\0" "kil1\0" "key_il\0" "kcub1\0" \
"key_left\0" "kll\0" "key_ll\0" "knp\0" "key_npage\0" "kpp\0" "key_ppage\0" \
"kcuf1\0" "key_right\0" "kind\0" "key_sf\0" "kri\0" "key_sr\0" "khts\0" \
"key_stab\0" "kcuu1\0" "key_up\0" "rmkx\0" "keypad_local\0" "smkx\0" "keypad_xmit\0" \
"lf0\0" "lab_f0\0" "lf1\0" "lab_f1\0" "lf10\0" "lab_f10\0" "lf2\0" "lab_f2\0" \
"lf3\0" "lab_f3\0" "lf4\0" "lab_f4\0" "lf5\0" "lab_f5\0" "lf6\0" "lab_f6\0" \
"lf7\0" "lab_f7\0" "lf8\0" "lab_f8\0" "lf9\0" "lab_f9\0" "rmm\0" "meta_off\0" \
"smm\0" "meta_on\0" "nel\0" "newline\0" "pad\0" "pad_char\0" "dch\0" "parm_dch\0" \
"dl\0" "parm_delete_line\0" "cud\0" "parm_down_cursor\0" "ich\0" "parm_ich\0" \
"indn\0" "parm_index\0" "il\0" "parm_insert_line\0" "cub\0" "parm_left_cursor\0" \
"cuf\0" "parm_right_cursor\0" "rin\0" "parm_rindex\0" "cuu\0" "parm_up_cursor\0" \
"pfkey\0" "pkey_key\0" "pfloc\0" "pkey_local\0" "pfx\0" "pkey_xmit\0" "mc0\0" \
"print_screen\0" "mc4\0" "prtr_off\0" "mc5\0" "prtr_on\0" "rep\0" "repeat_char\0" \
"rs1\0" "reset_1string\0" "rs2\0" "reset_2string\0" "rs3\0" "reset_3string\0" \
"rf\0" "reset_file\0" "rc\0" "restore_cursor\0" "vpa\0" "row_address\0" \
"sc\0" "save_cursor\0" "ind\0" "scroll_forward\0" "ri\0" "scroll_reverse\0" \
"sgr\0" "set_attributes\0" "hts\0" "set_tab\0" "wind\0" "set_window\0" \
"ht\0" "tab\0" "tsl\0" "to_status_line\0" "uc\0" "underline_char\0" "hu\0" \
"up_half_line\0" "iprog\0" "init_prog\0" "ka1\0" "key_a1\0" "ka3\0" "key_a3\0" \
"kb2\0" "key_b2\0" "kc1\0" "key_c1\0" "kc3\0" "key_c3\0" "mc5p\0" "prtr_non\0" \
"rmp\0" "char_padding\0" "acsc\0" "acs_chars\0" "pln\0" "plab_norm\0" "kcbt\0" \
"key_btab\0" "smxon\0" "enter_xon_mode\0" "rmxon\0" "exit_xon_mode\0" "smam\0" \
"enter_am_mode\0" "rmam\0" "exit_am_mode\0" "xonc\0" "xon_character\0" \
"xoffc\0" "xoff_character\0" "enacs\0" "ena_acs\0" "smln\0" "label_on\0" \
"rmln\0" "label_off\0" "kbeg\0" "key_beg\0" "kcan\0" "key_cancel\0" "kclo\0" \
"key_close\0" "kcmd\0" "key_command\0" "kcpy\0" "key_copy\0" "kcrt\0" "key_create\0" \
"kend\0" "key_end\0" "kent\0" "key_enter\0" "kext\0" "key_exit\0" "kfnd\0" \
"key_find\0" "khlp\0" "key_help\0" "kmrk\0" "key_mark\0" "kmsg\0" "key_message\0" \
"kmov\0" "key_move\0" "knxt\0" "key_next\0" "kopn\0" "key_open\0" "kopt\0" \
"key_options\0" "kprv\0" "key_previous\0" "kprt\0" "key_print\0" "krdo\0" \
"key_redo\0" "kref\0" "key_reference\0" "krfr\0" "key_refresh\0" "krpl\0" \
"key_replace\0" "krst\0" "key_restart\0" "kres\0" "key_resume\0" "ksav\0" \
"key_save\0" "kspd\0" "key_suspend\0" "kund\0" "key_undo\0" "kBEG\0" "key_sbeg\0" \
"kCAN\0" "key_scancel\0" "kCMD\0" "key_scommand\0" "kCPY\0" "key_scopy\0" \
"kCRT\0" "key_screate\0" "kDC\0" "key_sdc\0" "kDL\0" "key_sdl\0" "kslt\0" \
"key_select\0" "kEND\0" "key_send\0" "kEOL\0" "key_seol\0" "kEXT\0" "key_sexit\0" \
"kFND\0" "key_sfind\0" "kHLP\0" "key_shelp\0" "kHOM\0" "key_shome\0" "kIC\0" \
"key_sic\0" "kLFT\0" "key_sleft\0" "kMSG\0" "key_smessage\0" "kMOV\0" "key_smove\0" \
"kNXT\0" "key_snext\0" "kOPT\0" "key_soptions\0" "kPRV\0" "key_sprevious\0" \
"kPRT\0" "key_sprint\0" "kRDO\0" "key_sredo\0" "kRPL\0" "key_sreplace\0" \
"kRIT\0" "key_sright\0" "kRES\0" "key_srsume\0" "kSAV\0" "key_ssave\0" \
"kSPD\0" "key_ssuspend\0" "kUND\0" "key_sundo\0" "rfi\0" "req_for_input\0" \
"kf11\0" "key_f11\0" "kf12\0" "key_f12\0" "kf13\0" "key_f13\0" "kf14\0" \
"key_f14\0" "kf15\0" "key_f15\0" "kf16\0" "key_f16\0" "kf17\0" "key_f17\0" \
"kf18\0" "key_f18\0" "kf19\0" "key_f19\0" "kf20\0" "key_f20\0" "kf21\0" \
"key_f21\0" "kf22\0" "key_f22\0" "kf23\0" "key_f23\0" "kf24\0" "key_f24\0" \
"kf25\0" "key_f25\0" "kf26\0" "key_f26\0" "kf27\0" "key_f27\0" "kf28\0" \
"key_f28\0" "kf29\0" "key_f29\0" "kf30\0" "key_f30\0" "kf31\0" "key_f31\0" \
"kf32\0" "key_f32\0" "kf33\0" "key_f33\0" "kf34\0" "key_f34\0" "kf35\0" \
"key_f35\0" "kf36\0" "key_f36\0" "kf37\0" "key_f37\0" "kf38\0" "key_f38\0" \
"kf39\0" "key_f39\0" "kf40\0" "key_f40\0" "kf41\0" "key_f41\0" "kf42\0" \
"key_f42\0" "kf43\0" "key_f43\0" "kf44\0" "key_f44\0" "kf45\0" "key_f45\0" \
"kf46\0" "key_f46\0" "kf47\0" "key_f47\0" "kf48\0" "key_f48\0" "kf49\0" \
"key_f49\0" "kf50\0" "key_f50\0" "kf51\0" "key_f51\0" "kf52\0" "key_f52\0" \
"kf53\0" "key_f53\0" "kf54\0" "key_f54\0" "kf55\0" "key_f55\0" "kf56\0" \
"key_f56\0" "kf57\0" "key_f57\0" "kf58\0" "key_f58\0" "kf59\0" "key_f59\0" \
"kf60\0" "key_f60\0" "kf61\0" "key_f61\0" "kf62\0" "key_f62\0" "kf63\0" \
"key_f63\0" "el1\0" "clr_bol\0" "mgc\0" "clear_margins\0" "smgl\0" "set_left_margin\0" \
"smgr\0" "set_right_margin\0" "fln\0" "label_format\0" "sclk\0" "set_clock\0" \
"dclk\0" "display_clock\0" "rmclk\0" "remove_clock\0" "cwin\0" "create_window\0" \
"wingo\0" "goto_window\0" "hup\0" "hangup\0" "dial\0" "dial_phone\0" "qdial\0" \
"quick_dial\0" "tone\0" "tone\0" "pulse\0" "pulse\0" "hook\0" "flash_hook\0" \
"pause\0" "fixed_pause\0" "wait\0" "wait_tone\0" "u0\0" "user0\0" "u1\0" \
"user1\0" "u2\0" "user2\0" "u3\0" "user3\0" "u4\0" "user4\0" "u5\0" "user5\0" \
"u6\0" "user6\0" "u7\0" "user7\0" "u8\0" "user8\0" "u9\0" "user9\0" "op\0" \
"orig_pair\0" "oc\0" "orig_colors\0" "initc\0" "initialize_color\0" "initp\0" \
"initialize_pair\0" "scp\0" "set_color_pair\0" "setf\0" "set_foreground\0" \
"setb\0" "set_background\0" "cpi\0" "change_char_pitch\0" "lpi\0" "change_line_pitch\0" \
"chr\0" "change_res_horz\0" "cvr\0" "change_res_vert\0" "defc\0" "define_char\0" \
"swidm\0" "enter_doublewide_mode\0" "sdrfq\0" "enter_draft_quality\0" "sitm\0" \
"enter_italics_mode\0" "slm\0" "enter_leftward_mode\0" "smicm\0" "enter_micro_mode\0" \
"snlq\0" "enter_near_letter_quality\0" "snrmq\0" "enter_normal_quality\0" \
"sshm\0" "enter_shadow_mode\0" "ssubm\0" "enter_subscript_mode\0" "ssupm\0" \
"enter_superscript_mode\0" "sum\0" "enter_upward_mode\0" "rwidm\0" "exit_doublewide_mode\0" \
"ritm\0" "exit_italics_mode\0" "rlm\0" "exit_leftward_mode\0" "rmicm\0" \
"exit_micro_mode\0" "rshm\0" "exit_shadow_mode\0" "rsubm\0" "exit_subscript_mode\0" \
"rsupm\0" "exit_superscript_mode\0" "rum\0" "exit_upward_mode\0" "mhpa\0" \
"micro_column_address\0" "mcud1\0" "micro_down\0" "mcub1\0" "micro_left\0" \
"mcuf1\0" "micro_right\0" "mvpa\0" "micro_row_address\0" "mcuu1\0" "micro_up\0" \
"porder\0" "order_of_pins\0" "mcud\0" "parm_down_micro\0" "mcub\0" "parm_left_micro\0" \
"mcuf\0" "parm_right_micro\0" "mcuu\0" "parm_up_micro\0" "scs\0" "select_char_set\0" \
"smgb\0" "set_bottom_margin\0" "smgbp\0" "set_bottom_margin_parm\0" "smglp\0" \
"set_left_margin_parm\0" "smgrp\0" "set_right_margin_parm\0" "smgt\0" "set_top_margin\0" \
"smgtp\0" "set_top_margin_parm\0" "sbim\0" "start_bit_image\0" "scsd\0" \
"start_char_set_def\0" "rbim\0" "stop_bit_image\0" "rcsd\0" "stop_char_set_def\0" \
"subcs\0" "subscript_characters\0" "supcs\0" "superscript_characters\0" \
"docr\0" "these_cause_cr\0" "zerom\0" "zero_motion\0" "csnm\0" "char_set_names\0" \
"kmous\0" "key_mouse\0" "minfo\0" "mouse_info\0" "reqmp\0" "req_mouse_pos\0" \
"getm\0" "get_mouse\0" "setaf\0" "set_a_foreground\0" "setab\0" "set_a_background\0" \
"pfxl\0" "pkey_plab\0" "devt\0" "device_type\0" "csin\0" "code_set_init\0" \
"s0ds\0" "set0_des_seq\0" "s1ds\0" "set1_des_seq\0" "s2ds\0" "set2_des_seq\0" \
"s3ds\0" "set3_des_seq\0" "smglr\0" "set_lr_margin\0" "smgtb\0" "set_tb_margin\0" \
"birep\0" "bit_image_repeat\0" "binel\0" "bit_image_newline\0" "bicr\0" \
"bit_image_carriage_return\0" "colornm\0" "color_names\0" "defbi\0" "define_bit_image_region\0" \
"endbi\0" "end_bit_image_region\0" "setcolor\0" "set_color_band\0" "slines\0" \
"set_page_length\0" "dispc\0" "display_pc_char\0" "smpch\0" "enter_pc_charset_mode\0" \
"rmpch\0" "exit_pc_charset_mode\0" "smsc\0" "enter_scancode_mode\0" "rmsc\0" \
"exit_scancode_mode\0" "pctrm\0" "pc_term_options\0" "scesc\0" "scancode_escape\0" \
"scesa\0" "alt_scancode_esc\0" "ehhlm\0" "enter_horizontal_hl_mode\0" "elhlm\0" \
"enter_left_hl_mode\0" "elohlm\0" "enter_low_hl_mode\0" "erhlm\0" "enter_right_hl_mode\0" \
"ethlm\0" "enter_top_hl_mode\0" "evhlm\0" "enter_vertical_hl_mode\0" "sgr1\0" \
"set_a_attributes\0" "slength\0" "set_pglen_inch\0" "OTi2\0" "termcap_init2\0" \
"OTrs\0" "termcap_reset\0" "OTug\0" "magic_cookie_glitch_ul\0" "OTbs\0" \
"backspaces_with_bs\0" "OTns\0" "crt_no_scrolling\0" "OTnc\0" "no_correctly_working_cr\0" \
"OTdC\0" "carriage_return_delay\0" "OTdN\0" "new_line_delay\0" "OTnl\0" \
"linefeed_if_not_lf\0" "OTbc\0" "backspace_if_not_bs\0" "OTMT\0" "gnu_has_meta_key\0" \
"OTNL\0" "linefeed_is_newline\0" "OTdB\0" "backspace_delay\0" "OTdT\0" \
"horizontal_tab_delay\0" "OTkn\0" "number_of_function_keys\0" "OTko\0" \
"other_non_function_keys\0" "OTma\0" "arrow_key_map\0" "OTpt\0" "has_hardware_tabs\0" \
"OTxr\0" "return_does_clr_eol\0" "OTG2\0" "acs_ulcorner\0" "OTG3\0" "acs_llcorner\0" \
"OTG1\0" "acs_urcorner\0" "OTG4\0" "acs_lrcorner\0" "OTGR\0" "acs_ltee\0" \
"OTGL\0" "acs_rtee\0" "OTGU\0" "acs_btee\0" "OTGD\0" "acs_ttee\0" "OTGH\0" \
"acs_hline\0" "OTGV\0" "acs_vline\0" "OTGC\0" "acs_plus\0" "meml\0" "memory_lock\0" \
"memu\0" "memory_unlock\0" "box1\0" "box_chars_1\0" ;
static const short _nc_offset_boolfnames[] = {
3, 23, 45, 61, 87, 109, 129,
145, 158, 174, 193, 215, 231,
248, 270, 292, 310, 332, 356,
372, 398, 412, 432, 449, 467,
485, 503, 530, 545, 566, 596,
617, 645, 666, 686, 714, 735,
8193, 8217, 8239, 8364, 8386, 8535, 8558, };

static NCURSES_CONST char ** ptr_boolfnames = 0;

static const short _nc_offset_boolnames[] = {
0, 20, 41, 57, 82, 106, 126, 142, 155, 171, 190, 212, 228, 244, 265, 289, 304, 329, 353, 369, 394, 407, 427, 444, 461, 481, 497, 526, 541, 562, 591, 612, 639, 661, 682, 709, 730, 8188, 8212, 8234, 8359, 8381, 8530, 8553, };

static NCURSES_CONST char ** ptr_boolnames = 0;

static const short _nc_offset_numfnames[] = {
756, 767, 783, 792, 812, 835,
856, 877, 900, 914, 930, 945,
965, 988, 1005, 1019, 1040, 1062, 1085, 1108, 1132, 1151, 1170,
1192, 1211, 1231, 1252, 1278, 1303, 1320, 1340, 1355, 1382, 8165,
8268, 8295, 8411, 8432,
8458, };

static NCURSES_CONST char ** ptr_numfnames = 0;

static const short _nc_offset_numnames[] = {
751, 764, 777, 789, 808, 832, 853, 873, 895, 911, 927, 942, 960, 981, 999, 1015, 1034, 1056,
1079, 1102, 1126, 1147, 1166, 1186, 1207, 1227, 1247, 1273, 1299,
1314, 1335, 1348, 1375, 8160, 8263, 8290, 8406, 8427, 8453, };

static NCURSES_CONST char ** ptr_numnames = 0;

static const short _nc_offset_strfnames[] = {
1401,
1414, 1422, 1442, 1467, 1488, 1504, 1515, 1527, 1548, 1570, 1590,
1607, 1625, 1647, 1665, 1690, 1709, 1725, 1743, 1759, 1779, 1800,
1816, 1835, 1856, 1885, 1907, 1929, 1948, 1970, 1990, 2014, 2037,
2062, 2086, 2111, 2136, 2154, 2181, 2207, 2225, 2247, 2269, 2293,
2319, 2335, 2349, 2370, 2387, 2404, 2420, 2435, 2456, 2471, 2490,
2509, 2524, 2540, 2555, 2567, 2580, 2595, 2607, 2619, 2631, 2642,
2654, 2666, 2677, 2688, 2699, 2710, 2721, 2732, 2743, 2756, 2771,
2783, 2796, 2809, 2820, 2834, 2850, 2865, 2876, 2888, 2903, 2915,
2933, 2949, 2960, 2972, 2984, 2995, 3006, 3017, 3028, 3039, 3050,
3061, 3072, 3085, 3097, 3109, 3122, 3134, 3155, 3176, 3190, 3204,
3225, 3246, 3268, 3284, 3305, 3320, 3335, 3349, 3366, 3379, 3391,
3407, 3425, 3443, 3460, 3474, 3493, 3508, 3524, 3542, 3561, 3580,
3593, 3607, 3615, 3633, 3651, 3670, 3684, 3695, 3706, 3717, 3728,
3740, 3753, 3771, 3785, 3800, 3815, 3836, 3855, 3874, 3892, 3912,
3933, 3946, 3960, 3975, 3988, 4004, 4019, 4036, 4050, 4066, 4079,
4094, 4108, 4122, 4136, 4150, 4167, 4181, 4195, 4209, 4226, 4244,
4259, 4273, 4292, 4309, 4326, 4343, 4359, 4373, 4390, 4404, 4418,
4435, 4453, 4468, 4484, 4496, 4509, 4525, 4539, 4553, 4568, 4583,
4598, 4612, 4625, 4640, 4658, 4673, 4688, 4706, 4725, 4741, 4756,
4774, 4790, 4806, 4821, 4839, 4853, 4872, 4885, 4898, 4911, 4924,
4937, 4950, 4963, 4976, 4989, 5002, 5015, 5028, 5041, 5054, 5067,
5080, 5093, 5106, 5119, 5132, 5145, 5158, 5171, 5184, 5197, 5210,
5223, 5236, 5249, 5262, 5275, 5288, 5301, 5314, 5327, 5340, 5353,
5366, 5379, 5392, 5405, 5418, 5431, 5444, 5457, 5470, 5483, 5496,
5509, 5522, 5535, 5548, 5560, 5572, 5591, 5612, 5633, 5651, 5666,
5686, 5704, 5724, 5740, 5752, 5769, 5785, 5796, 5807, 5824, 5841,
5854, 5863, 5872, 5881, 5890, 5899, 5908, 5917, 5926, 5935, 5944,
5957, 5975, 5998, 6018, 6038, 6058, 6077, 6099, 6121, 6141, 6162,
6180, 6208, 6233, 6256, 6282, 6304, 6336, 6362, 6386, 6413, 6440,
6464, 6490, 6512, 6537, 6558, 6581, 6607, 6633, 6655, 6682, 6699,
6716, 6733, 6757, 6773, 6792, 6813, 6834, 6856, 6874, 6895, 6919,
6948, 6975, 7002, 7023, 7048, 7069, 7093, 7113, 7137, 7164, 7192,
7213, 7230, 7251, 7267, 7284, 7303, 7319, 7342, 7364, 7379, 7396,
7415, 7433, 7451, 7469, 7488, 7508, 7528, 7551, 7574, 7608, 7626,
7656, 7686, 7708, 7730, 7752, 7780, 7806, 7831, 7856, 7878, 7900,
7923, 7954, 7980, 8004, 8030, 8054, 8082, 8107, 8127, 8146, 8315, 8339, 8487, 8516, 8583, 8601, 8619, 8637, 8655, 8669,
8683, 8697, 8711, 8726, 8741, 8755, 8772, 8791, };

static NCURSES_CONST char ** ptr_strfnames = 0;

static const short _nc_offset_strnames[] = {
1397, 1410, 1419, 1438, 1463, 1482, 1501,
1512, 1523, 1542, 1566, 1585, 1602, 1619, 1642, 1659, 1684, 1704,
1722, 1738, 1753, 1774, 1796, 1812, 1832, 1850, 1879, 1902, 1923,
1943, 1966, 1985, 2008, 2032, 2058, 2081, 2106, 2132, 2148, 2176,
2201, 2220, 2242, 2264, 2288, 2313, 2332, 2345, 2366, 2383, 2400,
2417, 2430, 2452, 2468, 2486, 2504, 2519, 2534, 2549, 2562, 2574,
2589, 2603, 2615, 2627, 2638, 2649, 2662, 2673, 2684, 2695, 2706,
2717, 2728, 2739, 2750, 2765, 2778, 2790, 2805, 2816, 2830, 2844,
2860, 2872, 2883, 2897, 2910, 2928, 2945, 2956, 2967, 2980, 2991,
3002, 3013, 3024, 3035, 3046, 3057, 3068, 3081, 3093, 3105, 3118,
3131, 3151, 3172, 3185, 3201, 3221, 3242, 3264, 3280, 3299, 3314,
3331, 3345, 3362, 3375, 3387, 3403, 3421, 3439, 3457, 3471, 3489,
3505, 3520, 3539, 3557, 3576, 3588, 3604, 3611, 3630, 3648, 3664,
3680, 3691, 3702, 3713, 3724, 3735, 3749, 3766, 3781, 3795, 3809,
3830, 3850, 3869, 3887, 3906, 3927, 3941, 3955, 3970, 3983, 3999,
4014, 4031, 4045, 4061, 4074, 4089, 4103, 4117, 4131, 4145, 4162,
4176, 4190, 4204, 4221, 4239, 4254, 4268, 4287, 4304, 4321, 4338,
4354, 4368, 4385, 4399, 4413, 4430, 4448, 4463, 4480, 4492, 4504,
4520, 4534, 4548, 4563, 4578, 4593, 4608, 4620, 4635, 4653, 4668,
4683, 4701, 4720, 4736, 4751, 4769, 4785, 4801, 4816, 4834, 4849,
4867, 4880, 4893, 4906, 4919, 4932, 4945, 4958, 4971, 4984, 4997,
5010, 5023, 5036, 5049, 5062, 5075, 5088, 5101, 5114, 5127, 5140,
5153, 5166, 5179, 5192, 5205, 5218, 5231, 5244, 5257, 5270, 5283,
5296, 5309, 5322, 5335, 5348, 5361, 5374, 5387, 5400, 5413, 5426,
5439, 5452, 5465, 5478, 5491, 5504, 5517, 5530, 5543, 5556, 5568,
5586, 5607, 5629, 5646, 5661, 5680, 5699, 5718, 5736, 5747, 5763,
5780, 5790, 5802, 5818, 5836, 5851, 5860, 5869, 5878, 5887, 5896,
5905, 5914, 5923, 5932, 5941, 5954, 5969, 5992, 6014, 6033, 6053,
6073, 6095, 6117, 6137, 6157, 6174, 6202, 6228, 6252, 6276, 6299,
6330, 6357, 6380, 6407, 6436, 6458, 6485, 6508, 6531, 6553, 6575,
6601, 6629, 6650, 6676, 6693, 6710, 6728, 6751, 6766, 6787, 6808,
6829, 6851, 6870, 6890, 6913, 6942, 6969, 6997, 7017, 7043, 7064,
7088, 7108, 7131, 7158, 7187, 7207, 7225, 7245, 7261, 7278, 7298,
7313, 7336, 7359, 7374, 7391, 7410, 7428, 7446, 7464, 7482, 7502,
7522, 7545, 7569, 7600, 7620, 7650, 7677, 7701, 7724, 7746, 7774,
7801, 7826, 7850, 7872, 7894, 7917, 7948, 7973, 7998, 8024, 8048,
8077, 8099, 8122, 8141, 8310,
8334, 8482, 8511, 8578,
8596, 8614, 8632, 8650, 8664, 8678, 8692, 8706, 8721, 8736, 8750,
8767, 8786, };

static NCURSES_CONST char ** ptr_strnames = 0;


static IT *
alloc_array(NCURSES_CONST char ***value, const short *offsets, unsigned size)
{
	if (*value == 0) {
		if ((*value = typeCalloc(NCURSES_CONST char *, size + 1)) != 0) {
			unsigned n;
			for (n = 0; n < size; ++n) {
				(*value)[n] = (NCURSES_CONST char *) _nc_name_blob + offsets[n];
			}
		}
	}
	return *value;
}

#define FIX(it) NCURSES_IMPEXP IT * NCURSES_API NCURSES_PUBLIC_VAR(it)(void) { return alloc_array(&ptr_##it, _nc_offset_##it, SIZEOF(_nc_offset_##it)); }

/* remove public definition which conflicts with FIX() */
#undef boolnames
#undef boolfnames
#undef numnames
#undef numfnames
#undef strnames
#undef strfnames

/* add local definition */
FIX(boolnames)
FIX(boolfnames)
FIX(numnames)
FIX(numfnames)
FIX(strnames)
FIX(strfnames)

/* restore the public definition */
#define boolnames  NCURSES_PUBLIC_VAR(boolnames())
#define boolfnames NCURSES_PUBLIC_VAR(boolfnames())
#define numnames   NCURSES_PUBLIC_VAR(numnames())
#define numfnames  NCURSES_PUBLIC_VAR(numfnames())
#define strnames   NCURSES_PUBLIC_VAR(strnames())
#define strfnames  NCURSES_PUBLIC_VAR(strfnames())

#define FREE_FIX(it) if (ptr_##it) { FreeAndNull(ptr_##it); }

#if NO_LEAKS
NCURSES_EXPORT(void)
_nc_names_leaks(void)
{
FREE_FIX(boolnames)
FREE_FIX(boolfnames)
FREE_FIX(numnames)
FREE_FIX(numfnames)
FREE_FIX(strnames)
FREE_FIX(strfnames)
}
#endif

#else

#define DCL(it) NCURSES_EXPORT_VAR(IT) it[]

DCL(boolnames) = {
		"bw",
		"am",
		"xsb",
		"xhp",
		"xenl",
		"eo",
		"gn",
		"hc",
		"km",
		"hs",
		"in",
		"da",
		"db",
		"mir",
		"msgr",
		"os",
		"eslok",
		"xt",
		"hz",
		"ul",
		"xon",
		"nxon",
		"mc5i",
		"chts",
		"nrrmc",
		"npc",
		"ndscr",
		"ccc",
		"bce",
		"hls",
		"xhpa",
		"crxm",
		"daisy",
		"xvpa",
		"sam",
		"cpix",
		"lpix",
		"OTbs",
		"OTns",
		"OTnc",
		"OTMT",
		"OTNL",
		"OTpt",
		"OTxr",

		(NCURSES_CONST char *)0,
};

DCL(boolfnames) = {
		"auto_left_margin",
		"auto_right_margin",
		"no_esc_ctlc",
		"ceol_standout_glitch",
		"eat_newline_glitch",
		"erase_overstrike",
		"generic_type",
		"hard_copy",
		"has_meta_key",
		"has_status_line",
		"insert_null_glitch",
		"memory_above",
		"memory_below",
		"move_insert_mode",
		"move_standout_mode",
		"over_strike",
		"status_line_esc_ok",
		"dest_tabs_magic_smso",
		"tilde_glitch",
		"transparent_underline",
		"xon_xoff",
		"needs_xon_xoff",
		"prtr_silent",
		"hard_cursor",
		"non_rev_rmcup",
		"no_pad_char",
		"non_dest_scroll_region",
		"can_change",
		"back_color_erase",
		"hue_lightness_saturation",
		"col_addr_glitch",
		"cr_cancels_micro_mode",
		"has_print_wheel",
		"row_addr_glitch",
		"semi_auto_right_margin",
		"cpi_changes_res",
		"lpi_changes_res",
		"backspaces_with_bs",
		"crt_no_scrolling",
		"no_correctly_working_cr",
		"gnu_has_meta_key",
		"linefeed_is_newline",
		"has_hardware_tabs",
		"return_does_clr_eol",

		(NCURSES_CONST char *)0,
};

DCL(numnames) = {
		"cols",
		"it",
		"lines",
		"lm",
		"xmc",
		"pb",
		"vt",
		"wsl",
		"nlab",
		"lh",
		"lw",
		"ma",
		"wnum",
		"colors",
		"pairs",
		"ncv",
		"bufsz",
		"spinv",
		"spinh",
		"maddr",
		"mjump",
		"mcs",
		"mls",
		"npins",
		"orc",
		"orl",
		"orhi",
		"orvi",
		"cps",
		"widcs",
		"btns",
		"bitwin",
		"bitype",
		"OTug",
		"OTdC",
		"OTdN",
		"OTdB",
		"OTdT",
		"OTkn",

		(NCURSES_CONST char *)0,
};

DCL(numfnames) = {
		"columns",
		"init_tabs",
		"lines",
		"lines_of_memory",
		"magic_cookie_glitch",
		"padding_baud_rate",
		"virtual_terminal",
		"width_status_line",
		"num_labels",
		"label_height",
		"label_width",
		"max_attributes",
		"maximum_windows",
		"max_colors",
		"max_pairs",
		"no_color_video",
		"buffer_capacity",
		"dot_vert_spacing",
		"dot_horz_spacing",
		"max_micro_address",
		"max_micro_jump",
		"micro_col_size",
		"micro_line_size",
		"number_of_pins",
		"output_res_char",
		"output_res_line",
		"output_res_horz_inch",
		"output_res_vert_inch",
		"print_rate",
		"wide_char_size",
		"buttons",
		"bit_image_entwining",
		"bit_image_type",
		"magic_cookie_glitch_ul",
		"carriage_return_delay",
		"new_line_delay",
		"backspace_delay",
		"horizontal_tab_delay",
		"number_of_function_keys",

		(NCURSES_CONST char *)0,
};

DCL(strnames) = {
		"cbt",
		"bel",
		"cr",
		"csr",
		"tbc",
		"clear",
		"el",
		"ed",
		"hpa",
		"cmdch",
		"cup",
		"cud1",
		"home",
		"civis",
		"cub1",
		"mrcup",
		"cnorm",
		"cuf1",
		"ll",
		"cuu1",
		"cvvis",
		"dch1",
		"dl1",
		"dsl",
		"hd",
		"smacs",
		"blink",
		"bold",
		"smcup",
		"smdc",
		"dim",
		"smir",
		"invis",
		"prot",
		"rev",
		"smso",
		"smul",
		"ech",
		"rmacs",
		"sgr0",
		"rmcup",
		"rmdc",
		"rmir",
		"rmso",
		"rmul",
		"flash",
		"ff",
		"fsl",
		"is1",
		"is2",
		"is3",
		"if",
		"ich1",
		"il1",
		"ip",
		"kbs",
		"ktbc",
		"kclr",
		"kctab",
		"kdch1",
		"kdl1",
		"kcud1",
		"krmir",
		"kel",
		"ked",
		"kf0",
		"kf1",
		"kf10",
		"kf2",
		"kf3",
		"kf4",
		"kf5",
		"kf6",
		"kf7",
		"kf8",
		"kf9",
		"khome",
		"kich1",
		"kil1",
		"kcub1",
		"kll",
		"knp",
		"kpp",
		"kcuf1",
		"kind",
		"kri",
		"khts",
		"kcuu1",
		"rmkx",
		"smkx",
		"lf0",
		"lf1",
		"lf10",
		"lf2",
		"lf3",
		"lf4",
		"lf5",
		"lf6",
		"lf7",
		"lf8",
		"lf9",
		"rmm",
		"smm",
		"nel",
		"pad",
		"dch",
		"dl",
		"cud",
		"ich",
		"indn",
		"il",
		"cub",
		"cuf",
		"rin",
		"cuu",
		"pfkey",
		"pfloc",
		"pfx",
		"mc0",
		"mc4",
		"mc5",
		"rep",
		"rs1",
		"rs2",
		"rs3",
		"rf",
		"rc",
		"vpa",
		"sc",
		"ind",
		"ri",
		"sgr",
		"hts",
		"wind",
		"ht",
		"tsl",
		"uc",
		"hu",
		"iprog",
		"ka1",
		"ka3",
		"kb2",
		"kc1",
		"kc3",
		"mc5p",
		"rmp",
		"acsc",
		"pln",
		"kcbt",
		"smxon",
		"rmxon",
		"smam",
		"rmam",
		"xonc",
		"xoffc",
		"enacs",
		"smln",
		"rmln",
		"kbeg",
		"kcan",
		"kclo",
		"kcmd",
		"kcpy",
		"kcrt",
		"kend",
		"kent",
		"kext",
		"kfnd",
		"khlp",
		"kmrk",
		"kmsg",
		"kmov",
		"knxt",
		"kopn",
		"kopt",
		"kprv",
		"kprt",
		"krdo",
		"kref",
		"krfr",
		"krpl",
		"krst",
		"kres",
		"ksav",
		"kspd",
		"kund",
		"kBEG",
		"kCAN",
		"kCMD",
		"kCPY",
		"kCRT",
		"kDC",
		"kDL",
		"kslt",
		"kEND",
		"kEOL",
		"kEXT",
		"kFND",
		"kHLP",
		"kHOM",
		"kIC",
		"kLFT",
		"kMSG",
		"kMOV",
		"kNXT",
		"kOPT",
		"kPRV",
		"kPRT",
		"kRDO",
		"kRPL",
		"kRIT",
		"kRES",
		"kSAV",
		"kSPD",
		"kUND",
		"rfi",
		"kf11",
		"kf12",
		"kf13",
		"kf14",
		"kf15",
		"kf16",
		"kf17",
		"kf18",
		"kf19",
		"kf20",
		"kf21",
		"kf22",
		"kf23",
		"kf24",
		"kf25",
		"kf26",
		"kf27",
		"kf28",
		"kf29",
		"kf30",
		"kf31",
		"kf32",
		"kf33",
		"kf34",
		"kf35",
		"kf36",
		"kf37",
		"kf38",
		"kf39",
		"kf40",
		"kf41",
		"kf42",
		"kf43",
		"kf44",
		"kf45",
		"kf46",
		"kf47",
		"kf48",
		"kf49",
		"kf50",
		"kf51",
		"kf52",
		"kf53",
		"kf54",
		"kf55",
		"kf56",
		"kf57",
		"kf58",
		"kf59",
		"kf60",
		"kf61",
		"kf62",
		"kf63",
		"el1",
		"mgc",
		"smgl",
		"smgr",
		"fln",
		"sclk",
		"dclk",
		"rmclk",
		"cwin",
		"wingo",
		"hup",
		"dial",
		"qdial",
		"tone",
		"pulse",
		"hook",
		"pause",
		"wait",
		"u0",
		"u1",
		"u2",
		"u3",
		"u4",
		"u5",
		"u6",
		"u7",
		"u8",
		"u9",
		"op",
		"oc",
		"initc",
		"initp",
		"scp",
		"setf",
		"setb",
		"cpi",
		"lpi",
		"chr",
		"cvr",
		"defc",
		"swidm",
		"sdrfq",
		"sitm",
		"slm",
		"smicm",
		"snlq",
		"snrmq",
		"sshm",
		"ssubm",
		"ssupm",
		"sum",
		"rwidm",
		"ritm",
		"rlm",
		"rmicm",
		"rshm",
		"rsubm",
		"rsupm",
		"rum",
		"mhpa",
		"mcud1",
		"mcub1",
		"mcuf1",
		"mvpa",
		"mcuu1",
		"porder",
		"mcud",
		"mcub",
		"mcuf",
		"mcuu",
		"scs",
		"smgb",
		"smgbp",
		"smglp",
		"smgrp",
		"smgt",
		"smgtp",
		"sbim",
		"scsd",
		"rbim",
		"rcsd",
		"subcs",
		"supcs",
		"docr",
		"zerom",
		"csnm",
		"kmous",
		"minfo",
		"reqmp",
		"getm",
		"setaf",
		"setab",
		"pfxl",
		"devt",
		"csin",
		"s0ds",
		"s1ds",
		"s2ds",
		"s3ds",
		"smglr",
		"smgtb",
		"birep",
		"binel",
		"bicr",
		"colornm",
		"defbi",
		"endbi",
		"setcolor",
		"slines",
		"dispc",
		"smpch",
		"rmpch",
		"smsc",
		"rmsc",
		"pctrm",
		"scesc",
		"scesa",
		"ehhlm",
		"elhlm",
		"elohlm",
		"erhlm",
		"ethlm",
		"evhlm",
		"sgr1",
		"slength",
		"OTi2",
		"OTrs",
		"OTnl",
		"OTbc",
		"OTko",
		"OTma",
		"OTG2",
		"OTG3",
		"OTG1",
		"OTG4",
		"OTGR",
		"OTGL",
		"OTGU",
		"OTGD",
		"OTGH",
		"OTGV",
		"OTGC",
		"meml",
		"memu",
		"box1",

		(NCURSES_CONST char *)0,
};

DCL(strfnames) = {
		"back_tab",
		"bell",
		"carriage_return",
		"change_scroll_region",
		"clear_all_tabs",
		"clear_screen",
		"clr_eol",
		"clr_eos",
		"column_address",
		"command_character",
		"cursor_address",
		"cursor_down",
		"cursor_home",
		"cursor_invisible",
		"cursor_left",
		"cursor_mem_address",
		"cursor_normal",
		"cursor_right",
		"cursor_to_ll",
		"cursor_up",
		"cursor_visible",
		"delete_character",
		"delete_line",
		"dis_status_line",
		"down_half_line",
		"enter_alt_charset_mode",
		"enter_blink_mode",
		"enter_bold_mode",
		"enter_ca_mode",
		"enter_delete_mode",
		"enter_dim_mode",
		"enter_insert_mode",
		"enter_secure_mode",
		"enter_protected_mode",
		"enter_reverse_mode",
		"enter_standout_mode",
		"enter_underline_mode",
		"erase_chars",
		"exit_alt_charset_mode",
		"exit_attribute_mode",
		"exit_ca_mode",
		"exit_delete_mode",
		"exit_insert_mode",
		"exit_standout_mode",
		"exit_underline_mode",
		"flash_screen",
		"form_feed",
		"from_status_line",
		"init_1string",
		"init_2string",
		"init_3string",
		"init_file",
		"insert_character",
		"insert_line",
		"insert_padding",
		"key_backspace",
		"key_catab",
		"key_clear",
		"key_ctab",
		"key_dc",
		"key_dl",
		"key_down",
		"key_eic",
		"key_eol",
		"key_eos",
		"key_f0",
		"key_f1",
		"key_f10",
		"key_f2",
		"key_f3",
		"key_f4",
		"key_f5",
		"key_f6",
		"key_f7",
		"key_f8",
		"key_f9",
		"key_home",
		"key_ic",
		"key_il",
		"key_left",
		"key_ll",
		"key_npage",
		"key_ppage",
		"key_right",
		"key_sf",
		"key_sr",
		"key_stab",
		"key_up",
		"keypad_local",
		"keypad_xmit",
		"lab_f0",
		"lab_f1",
		"lab_f10",
		"lab_f2",
		"lab_f3",
		"lab_f4",
		"lab_f5",
		"lab_f6",
		"lab_f7",
		"lab_f8",
		"lab_f9",
		"meta_off",
		"meta_on",
		"newline",
		"pad_char",
		"parm_dch",
		"parm_delete_line",
		"parm_down_cursor",
		"parm_ich",
		"parm_index",
		"parm_insert_line",
		"parm_left_cursor",
		"parm_right_cursor",
		"parm_rindex",
		"parm_up_cursor",
		"pkey_key",
		"pkey_local",
		"pkey_xmit",
		"print_screen",
		"prtr_off",
		"prtr_on",
		"repeat_char",
		"reset_1string",
		"reset_2string",
		"reset_3string",
		"reset_file",
		"restore_cursor",
		"row_address",
		"save_cursor",
		"scroll_forward",
		"scroll_reverse",
		"set_attributes",
		"set_tab",
		"set_window",
		"tab",
		"to_status_line",
		"underline_char",
		"up_half_line",
		"init_prog",
		"key_a1",
		"key_a3",
		"key_b2",
		"key_c1",
		"key_c3",
		"prtr_non",
		"char_padding",
		"acs_chars",
		"plab_norm",
		"key_btab",
		"enter_xon_mode",
		"exit_xon_mode",
		"enter_am_mode",
		"exit_am_mode",
		"xon_character",
		"xoff_character",
		"ena_acs",
		"label_on",
		"label_off",
		"key_beg",
		"key_cancel",
		"key_close",
		"key_command",
		"key_copy",
		"key_create",
		"key_end",
		"key_enter",
		"key_exit",
		"key_find",
		"key_help",
		"key_mark",
		"key_message",
		"key_move",
		"key_next",
		"key_open",
		"key_options",
		"key_previous",
		"key_print",
		"key_redo",
		"key_reference",
		"key_refresh",
		"key_replace",
		"key_restart",
		"key_resume",
		"key_save",
		"key_suspend",
		"key_undo",
		"key_sbeg",
		"key_scancel",
		"key_scommand",
		"key_scopy",
		"key_screate",
		"key_sdc",
		"key_sdl",
		"key_select",
		"key_send",
		"key_seol",
		"key_sexit",
		"key_sfind",
		"key_shelp",
		"key_shome",
		"key_sic",
		"key_sleft",
		"key_smessage",
		"key_smove",
		"key_snext",
		"key_soptions",
		"key_sprevious",
		"key_sprint",
		"key_sredo",
		"key_sreplace",
		"key_sright",
		"key_srsume",
		"key_ssave",
		"key_ssuspend",
		"key_sundo",
		"req_for_input",
		"key_f11",
		"key_f12",
		"key_f13",
		"key_f14",
		"key_f15",
		"key_f16",
		"key_f17",
		"key_f18",
		"key_f19",
		"key_f20",
		"key_f21",
		"key_f22",
		"key_f23",
		"key_f24",
		"key_f25",
		"key_f26",
		"key_f27",
		"key_f28",
		"key_f29",
		"key_f30",
		"key_f31",
		"key_f32",
		"key_f33",
		"key_f34",
		"key_f35",
		"key_f36",
		"key_f37",
		"key_f38",
		"key_f39",
		"key_f40",
		"key_f41",
		"key_f42",
		"key_f43",
		"key_f44",
		"key_f45",
		"key_f46",
		"key_f47",
		"key_f48",
		"key_f49",
		"key_f50",
		"key_f51",
		"key_f52",
		"key_f53",
		"key_f54",
		"key_f55",
		"key_f56",
		"key_f57",
		"key_f58",
		"key_f59",
		"key_f60",
		"key_f61",
		"key_f62",
		"key_f63",
		"clr_bol",
		"clear_margins",
		"set_left_margin",
		"set_right_margin",
		"label_format",
		"set_clock",
		"display_clock",
		"remove_clock",
		"create_window",
		"goto_window",
		"hangup",
		"dial_phone",
		"quick_dial",
		"tone",
		"pulse",
		"flash_hook",
		"fixed_pause",
		"wait_tone",
		"user0",
		"user1",
		"user2",
		"user3",
		"user4",
		"user5",
		"user6",
		"user7",
		"user8",
		"user9",
		"orig_pair",
		"orig_colors",
		"initialize_color",
		"initialize_pair",
		"set_color_pair",
		"set_foreground",
		"set_background",
		"change_char_pitch",
		"change_line_pitch",
		"change_res_horz",
		"change_res_vert",
		"define_char",
		"enter_doublewide_mode",
		"enter_draft_quality",
		"enter_italics_mode",
		"enter_leftward_mode",
		"enter_micro_mode",
		"enter_near_letter_quality",
		"enter_normal_quality",
		"enter_shadow_mode",
		"enter_subscript_mode",
		"enter_superscript_mode",
		"enter_upward_mode",
		"exit_doublewide_mode",
		"exit_italics_mode",
		"exit_leftward_mode",
		"exit_micro_mode",
		"exit_shadow_mode",
		"exit_subscript_mode",
		"exit_superscript_mode",
		"exit_upward_mode",
		"micro_column_address",
		"micro_down",
		"micro_left",
		"micro_right",
		"micro_row_address",
		"micro_up",
		"order_of_pins",
		"parm_down_micro",
		"parm_left_micro",
		"parm_right_micro",
		"parm_up_micro",
		"select_char_set",
		"set_bottom_margin",
		"set_bottom_margin_parm",
		"set_left_margin_parm",
		"set_right_margin_parm",
		"set_top_margin",
		"set_top_margin_parm",
		"start_bit_image",
		"start_char_set_def",
		"stop_bit_image",
		"stop_char_set_def",
		"subscript_characters",
		"superscript_characters",
		"these_cause_cr",
		"zero_motion",
		"char_set_names",
		"key_mouse",
		"mouse_info",
		"req_mouse_pos",
		"get_mouse",
		"set_a_foreground",
		"set_a_background",
		"pkey_plab",
		"device_type",
		"code_set_init",
		"set0_des_seq",
		"set1_des_seq",
		"set2_des_seq",
		"set3_des_seq",
		"set_lr_margin",
		"set_tb_margin",
		"bit_image_repeat",
		"bit_image_newline",
		"bit_image_carriage_return",
		"color_names",
		"define_bit_image_region",
		"end_bit_image_region",
		"set_color_band",
		"set_page_length",
		"display_pc_char",
		"enter_pc_charset_mode",
		"exit_pc_charset_mode",
		"enter_scancode_mode",
		"exit_scancode_mode",
		"pc_term_options",
		"scancode_escape",
		"alt_scancode_esc",
		"enter_horizontal_hl_mode",
		"enter_left_hl_mode",
		"enter_low_hl_mode",
		"enter_right_hl_mode",
		"enter_top_hl_mode",
		"enter_vertical_hl_mode",
		"set_a_attributes",
		"set_pglen_inch",
		"termcap_init2",
		"termcap_reset",
		"linefeed_if_not_lf",
		"backspace_if_not_bs",
		"other_non_function_keys",
		"arrow_key_map",
		"acs_ulcorner",
		"acs_llcorner",
		"acs_urcorner",
		"acs_lrcorner",
		"acs_ltee",
		"acs_rtee",
		"acs_btee",
		"acs_ttee",
		"acs_hline",
		"acs_vline",
		"acs_plus",
		"memory_lock",
		"memory_unlock",
		"box_chars_1",

		(NCURSES_CONST char *)0,
};


#endif /* BROKEN_LINKER */
